# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/06_export_to_standalone.ipynb

__all__ = ['helper_one', 'helper_two']

# %% nbs/06_export_to_standalone.ipynb 2
def helper_one():
    """A helper function exported to standalone_utils module."""
    return "helper one"

# %% nbs/06_export_to_standalone.ipynb 3
def helper_two():
    """Another helper exported to the same module."""
    return "helper two"
