# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/06_export_to_standalone.ipynb

__all__ = ['from_separate']

# %% nbs/06_export_to_standalone.ipynb 4
def from_separate():
    """This goes to a completely separate module."""
    return "from separate"
