# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/04_documentation.ipynb

__all__ = ['public_api']

# %% nbs/04_documentation.ipynb 4
def _private_setup():
    """This function is exported but hidden from docs.

    Useful for internal helpers that shouldn't clutter documentation.
    """
    return "internal setup complete"

# %% nbs/04_documentation.ipynb 10
def public_api():
    """This function is exported AND visible in docs."""
    return _private_setup()
