# AUTOGENERATED! DO NOT EDIT! File to edit: pcts/core.pct.py

__all__ = ['VERSION', 'get_version', 'hello']

VERSION = "1.0.0"

def hello() -> str:
    """Say hello."""
    return "Hello from core!"

def get_version() -> str:
    """Get the version string."""
    return VERSION
