# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/utils.ipynb

__all__ = ['add', 'greet']

# %% nbs/utils.ipynb 2
def greet(name: str) -> str:
    """Greet someone."""
    return f"Hello, {name}!"

# %% nbs/utils.ipynb 3
def add(a: int, b: int) -> int:
    """Add two numbers."""
    return a + b
