# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/versioned.ipynb

__all__ = ['hello']

# %% nbs/versioned.ipynb 2
__author__ = "Jane Doe"
__version__ = "1.2.3"

# %% nbs/versioned.ipynb 3
def hello():
    """Say hello."""
    return f"Hello from versioned v{__version__}!"
