# AUTOGENERATED! DO NOT EDIT!

DEFAULT_MULTIPLIER = 2

def process_data(input_list: list, multiplier: int = 2) -> list:
    # Validate input
    if not isinstance(input_list, list):
        raise TypeError("input_list must be a list")
    # Process the data
    result = [x * multiplier for x in input_list]
    return result
    return result
